/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2024-2024. All rights reserved.
 * Author: lu
 */
#ifndef OCK_HCOM_CGO_H
#define OCK_HCOM_CGO_H

#include "hcom_c.h"
#include "hcom_service_c.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * @brief msg information
 */
typedef struct {
    void *data;        /* msg data */
    uint64_t dataSize; /* msg size */
} Msg_Info;

/*
 * @brief malloc memory
 *
 * @param size             [in] memory size
 *
 * @return allocated memory
 */
void *Cgo_AllocMem(uint32_t size);

/*
 * @brief memalign memory
 *
 * @param alignment        [in] alignment size
 * @param size             [in] memory size
 *
 * @return allocated memory
 */
void *Cgo_AlignMem(uint32_t alignment, uint64_t size);

/*
 * @brief free memory
 *
 * @param mem             [in] memory size
 */
void Cgo_FreeMem(void *mem);

/*
 * @brief memory copy
 *
 * @param dest            [in] destination memory address
 * @param src             [in] source memory address
 * @param destSize        [in] destination memory size
 * @param srcSize         [in] source memory size
 *
 * @return 0 means success
 */
int Cgo_MemoryCopy(void *dest, void *src, uint32_t destSize, uint32_t srcSize);

/*
 * @brief memory copy
 *
 * @param dest            [in] destination string
 * @param size            [in] destination string size
 * @param src             [in] source string
 *
 * @return 0 means success
 */
int Cgo_StringCopy(char dest[], uint32_t size, const char *src);

/*
 * @brief get rndv msg, includes data and length
 *
 * @param context          [in] rndv context
 * @param context          [out] msg
 *
 * @return 0 means success
 */
int Cgo_GetRndvMessage(Service_RndvContext context, Service_Message *msg);

/*
 * @brief get encrypt msg
 *
 * @param channel          [in] channel
 * @param rawData          [in] raw data
 * @param rawLen           [in] raw data length
 * @param mrAddress        [in] encrypt data put in mr address
 * @param mrSize           [in] mr address size
 *
 * @return encrypt data length, 0 means failed
 */
uint64_t Cgo_GetEncryptMsg(Net_Channel channel, void *rawData, uint64_t rawLen, uintptr_t mrAddress, uint64_t mrSize);

/*
 * @brief get decrypt msg
 *
 * @param channel          [in] channel
 * @param cipher           [in] encrypt data
 * @param cipherLen        [in] encrypt data length
 * @param rawData          [in] raw data
 * @param rawLen           [in] raw data length
 *
 * @return actual raw data length, 0 means failed
 */
uint64_t Cgo_GetDecryptMsg(Net_Channel channel, uintptr_t cipher, uint64_t cipherLen, void *rawData, uint64_t rawLen);

#ifdef __cplusplus
}
#endif

#endif // OCK_HCOM_CGO_H